#!/bin/sh

#
# Script to support generation of a "cluster" file on
# VIDM Connector instances for the purpose of migration
# to Windows.
#

. /usr/local/horizon/scripts/utils.inc
. /usr/local/horizon/scripts/manageTcCfg.inc
. /usr/local/horizon/scripts/flag.inc

RUNTIME_CONFIG=/usr/local/horizon/conf/runtime-config.properties
CLUSTER_DIR=/usr/local/horizon/conf/cluster
MIN_PWD_LEN=8

if test -z "${JAVA_HOME}" ; then
  export JAVA_HOME=/usr/java/jre-vmware
fi

SCRIPT_DIR=`dirname "$0"`
cd $SCRIPT_DIR
export SCRIPT_DIR=`pwd`

function usage() {
    echo "------------------------------------------------------------------------------"
    echo "Creates an encrypted data file that includes the files required for clustering"
    echo "including encryption keys. This file is protected by a strong password that is"
    echo "created by the user and stored under path provided."
    echo
    echo "Usage :"
    echo "   ./generateClusterFile.sh [<PASSWORD>] "
    echo
    echo "PARAMETERS:"
    echo
    echo "PASSWORD   (Optional) Password to use to encrypt or decrypt the cluster bundle."
    echo "           Use of a strong password, with a minimum of $MIN_PWD_LEN characters and"
    echo "           including numbers, symbols, capital letters, and lower-case letters"
    echo "           is recommended."
    echo
    echo "           If a password is not supplied, it will be prompted for."
    echo
    echo "Example:"
    echo "  generateClusterFile.bat password "
    echo "          creates an encrypted data file in the same directory as the script"
    echo "  ${SCRIPT_DIR}/cluster-<hostname>-conn-<timestamp>.enc"
    echo
    echo "------------------------------------------------------------------------------"
}

function getSyslogSettings(){
    local syslogFile=$1
    local SYSLOG_OPTS=

    local syslog_status=($(/usr/local/horizon/scripts/enableSyslog.hzn status))
    local SYSLOG_ENABLED=
    local SYSLOG_HOST=
    for kv in "${syslog_status[@]}"; do
        if test "${kv%%=*}" = "syslog_enabled"; then
          SYSLOG_ENABLED=${kv##*=}
        fi
        if test "${kv%%=*}" = "syslog_host"; then
          SYSLOG_HOST=${kv##*=}
        fi
    done

    if test "$SYSLOG_ENABLED" = "true"; then
      SYSLOG_OPTS="-Didm.syslog.host=${SYSLOG_HOST}"
    fi

    rm -f $syslogFile
    if test -n "$SYSLOG_OPTS"; then
        echo "set \"SYSLOG_OPTS=${SYSLOG_OPTS}\"" > $syslogFile
    fi
}

function isValidPassword(){
    if test -z "${1}" -o ${#1} -lt $MIN_PWD_LEN ; then
        echo "Password with a minimum length of $MIN_PWD_LEN required for cluster file generation." >&2
        return 1
    fi

  return 0
}

function getProxySettings(){
    local proxyFile=$1

    # If a proxy is configured (PROXY_ENABLED="yes"), we expect a link like the following
    # in the /etc/sysconfig/proxy file
    # HTTP_PROXY=http://proxy.example.com:3128
    local PROXY=( `grep -vE "^#" /etc/sysconfig/proxy | tr -d " \""` )
    local PROXY_ENABLED=
    local HTTP_PROXY=
    local HTTPS_PROXY=
    local NO_PROXY=
    local HTTP_NOPROXY=
    local HTTPS_NOPROXY=
    local PROXY_OPTS=

    for s in ${PROXY[@]}; do
        case ${s%=*} in
            PROXY_ENABLED)
              PROXY_ENABLED=${s##*=}
              ;;
            HTTP_PROXY)
              if [ -n "${s##*=}" ] ; then
                  HTTP_PROXY=${s##*://}
                  HTTP_PROXY=${HTTP_PROXY%/*}
                  HTTP_PROXY_USERNAME_PASSWORD=${HTTP_PROXY%@*}
                  if [ "${HTTP_PROXY_USERNAME_PASSWORD}" != "${HTTP_PROXY}" ] ; then
                      HTTP_PROXY_USERNAME=${HTTP_PROXY_USERNAME_PASSWORD%:*}
                      HTTP_PROXY_PASSWORD=${HTTP_PROXY_USERNAME_PASSWORD##*:}
                      HTTP_PROXY=${HTTP_PROXY##*@}
                  fi
              fi
              ;;
            HTTPS_PROXY)
              if [ -n "${s##*=}" ] ; then
                  HTTPS_PROXY=${s##*://}
                  HTTPS_PROXY=${HTTPS_PROXY%/*}
                  HTTPS_PROXY_USERNAME_PASSWORD=${HTTPS_PROXY%@*}
                  if [ "${HTTPS_PROXY_USERNAME_PASSWORD}" != "${HTTPS_PROXY}" ] ; then
                      HTTPS_PROXY_USERNAME=${HTTPS_PROXY_USERNAME_PASSWORD%:*}
                      HTTPS_PROXY_PASSWORD=${HTTPS_PROXY_USERNAME_PASSWORD##*:}
                      HTTPS_PROXY=${HTTPS_PROXY##*@}
                  fi
               fi
              ;;
            NO_PROXY)
              NO_PROXY=${s##*=}
              ;;
        esac
    done

    if test "${PROXY_ENABLED}" = "yes" ; then
        if [ -n "${NO_PROXY}" ] ; then
              NO_PROXY=${NO_PROXY//",."/"|*."}
              NO_PROXY=${NO_PROXY//","/"|"}
              HTTP_NOPROXY="-Dhttp.nonProxyHosts=${NO_PROXY}"
              HTTPS_NOPROXY="-Dhttps.nonProxyHosts=${NO_PROXY}"
        fi
        if [ -n "${HTTP_PROXY}" ] ; then
            PROXY_HOST="${HTTP_PROXY%:*}"
            PROXY_PORT="${HTTP_PROXY##*:}"
            PROXY_OPTS="${HTTP_NOPROXY} ${PROXY_OPTS}"
            if test "${PROXY_HOST}" != "${PROXY_PORT}"; then
                PROXY_OPTS="-Dhttp.proxyPort=${PROXY_PORT} ${PROXY_OPTS}"
            fi
            if [ -n "${HTTP_PROXY_USERNAME}" ] ; then
                PROXY_OPTS="-Dhttp.proxyUser=${HTTP_PROXY_USERNAME} ${PROXY_OPTS}"
            fi
            PROXY_OPTS="-Dhttp.proxyHost=${PROXY_HOST} ${PROXY_OPTS}"
        fi
        if [ -n "${HTTPS_PROXY}" ] ; then
            PROXY_HOST="${HTTPS_PROXY%:*}"
            PROXY_PORT="${HTTPS_PROXY##*:}"
            PROXY_OPTS="${HTTPS_NOPROXY} ${PROXY_OPTS}"
            if test "${PROXY_HOST}" != "${PROXY_PORT}"; then
                PROXY_OPTS="-Dhttps.proxyPort=${PROXY_PORT} ${PROXY_OPTS}"
            fi
            if [ -n "${HTTPS_PROXY_USERNAME}" ] ; then
                PROXY_OPTS="-Dhttps.proxyUser=${HTTPS_PROXY_USERNAME} ${PROXY_OPTS}"
            fi
            PROXY_OPTS="-Dhttps.proxyHost=${PROXY_HOST} ${PROXY_OPTS}"
        fi
    fi

    rm -f $proxyFile
    if test -n "$PROXY_OPTS"; then
        echo "set \"PROXY_OPTS=${PROXY_OPTS}\"" > $proxyFile
    fi
}

function generateMetaData(){
    local version=$(getAppVersion hc | tr -d '\r\n')
    echo -en "host=${hostname}\r\n" >> "${CLUSTER_DIR}/conf/cluster-info.txt"
    echo -en "ip=$(myip)\r\n" >> "${CLUSTER_DIR}/conf/cluster-info.txt"
    echo -en "date=$(date "+%Y-%m-%d_%H:%M:%S")\r\n" >> "${CLUSTER_DIR}/conf/cluster-info.txt"
    echo -en "version=${version}\r\n" >> "${CLUSTER_DIR}/conf/cluster-info.txt"
    echo -en "platform=unix\r\n" >> "${CLUSTER_DIR}/conf/cluster-info.txt"
}

function cleanup() {
    local rc=$1
    if test -e "${ZIP_FILE_NAME_WITH_PATH}" ; then
      rm -f "${ZIP_FILE_NAME_WITH_PATH}"
    fi

    if test -e "${CLUSTER_DIR}" ; then
      rm -fr "${CLUSTER_DIR}"
    fi
    exit $rc
}

FILE_ENC_JAR=$(ls -1r ${SCRIPT_DIR}/file-encrypter-*-jar-with-dependencies.jar | head -1)
if [ -z "$FILE_ENC_JAR" -o ! -e "$FILE_ENC_JAR" ]; then
  echo "File encryption jar file not located."
  echo "The provided file-encrypter jar needs to be in the same directory as this script."
  cleanup 2
fi

BC_JAR=$(ls -1r /usr/local/horizon/jre-endorsed/bc-fips*jar | head -1)
if [ -z "$BC_JAR" -o ! -e "$BC_JAR" ]; then
  echo "BouncyCastle FIPS jar not located."
  cleanup 2
fi

CLUSTER_PASSWORD=$1

if test -z "$CLUSTER_PASSWORD" ; then
    usage
    echo
    echo Password not supplied.  Collecting password to protect the cluster bundle.
    echo Password must have a minimum length of $MIN_PWD_LEN characters.
    echo
fi

while [ -z "$CLUSTER_PASSWORD" ]; do
    unset CLUSTER_PASSWORD_1
    unset CLUSTER_PASSWORD_2

    echo -n "Password: "
    read -s CLUSTER_PASSWORD_1
    echo
    if $(isValidPassword $CLUSTER_PASSWORD_1) ; then
        echo -n "Password (again): "
        read -s CLUSTER_PASSWORD_2
        echo
        if [ "$CLUSTER_PASSWORD_1" = "$CLUSTER_PASSWORD_2" ] ; then
            CLUSTER_PASSWORD="$CLUSTER_PASSWORD_1"
        else
            echo "**** Entered passwords do not match."
        fi
    fi
done

if ! $(isValidPassword $CLUSTER_PASSWORD) ; then
    cleanup 2
fi

hostname=$(getPropertyVal configurator.hostname ${RUNTIME_CONFIG})

ZIP_FILE_NAME="cluster-${hostname}-conn-$(date "+%s")"
ZIP_FILE_NAME_WITH_PATH=${SCRIPT_DIR}/${ZIP_FILE_NAME}
ENCRYPTED_ZIP_FILE=${ZIP_FILE_NAME_WITH_PATH}.enc

# cleanup before starting
rm -f "${ZIP_FILE_NAME_WITH_PATH}";
rm -f "${ENCRYPTED_ZIP_FILE}";
rm -fr ${CLUSTER_DIR}

echo "--- Creating cluster structure ---"
mkdir -p ${CLUSTER_DIR}/conf
mkdir -p ${CLUSTER_DIR}/workspace/bin
mkdir -p ${CLUSTER_DIR}/workspace/conf

echo "--- Generating cluster metadata info ---"
generateMetaData

echo "--- Gathering cluster settings ---"
cp /usr/local/horizon/conf/config*.* "${CLUSTER_DIR}/conf"

#convert root ca to windows format
export ROOT_CA_PASS=vmware
/usr/bin/openssl pkcs12 -export -in /usr/local/horizon/conf/root_ca.pem -inkey /usr/local/horizon/conf/root_ca_key.pem -name ca -out "${CLUSTER_DIR}/conf/root_ca.p12" -password env:ROOT_CA_PASS
cp /usr/local/horizon/conf/root_ca.pem "${CLUSTER_DIR}/conf/root_ca.cer"

if ! $(isConnectorOnly); then
  CLUSTER_RUNTIME_CONFIG=${CLUSTER_DIR}/conf/runtime-config.properties
  echo >  ${CLUSTER_RUNTIME_CONFIG}

        MIGRATION_PROPERTIES=( gateway.hostname
                               gateway.port
                               connector.api.version )
        for prop in ${MIGRATION_PROPERTIES[@]} ; do
          echo updating property: $prop
          val=$(getPropertyVal "${prop}" ${RUNTIME_CONFIG})
          replaceProperty "${prop}" "${val}" ${CLUSTER_RUNTIME_CONFIG}
        done
else
  cp /usr/local/horizon/conf/runtime-config.properties "${CLUSTER_DIR}/conf"
fi

cp /usr/local/horizon/conf/system-config.properties "${CLUSTER_DIR}/conf"

if test -e /usr/local/horizon/conf/idm-cacerts; then
    cp /usr/local/horizon/conf/idm-cacerts "${CLUSTER_DIR}/conf"
else
    cp ${JAVA_HOME}/lib/security/cacerts "${CLUSTER_DIR}/conf/idm-cacerts"
fi
cp /usr/local/horizon/conf/cakeystore.pass "${CLUSTER_DIR}/conf"

if test -e  /usr/local/horizon/conf/states; then
    cp -R /usr/local/horizon/conf/states "${CLUSTER_DIR}/conf"/states
fi

FLAG_DIR=/usr/local/horizon/conf/flags
if ! $(isConnectorOnly); then
    mkdir -p ${CLUSTER_DIR}/conf/flags/
    FLAG_PROP=( fips.mode
                sysconfig.datacenterid
                sysconfig.telemetry )
    for prop in ${FLAG_PROP[@]} ; do
       if [ -f $FLAG_DIR/$prop ]; then
         cp $FLAG_DIR/$prop ${CLUSTER_DIR}/conf/flags/
       fi
    done
  else
    cp -R $FLAG_DIR "${CLUSTER_DIR}/conf"/flags
fi

cp ${INSTANCE_KEYSTORE} "${CLUSTER_DIR}/workspace/conf"
cp ${TOMCAT_INSTANCE}/conf/catalina.properties "${CLUSTER_DIR}/workspace/conf"

echo "--- Processing syslog settings ---"
getSyslogSettings "${CLUSTER_DIR}/workspace/bin/syslogSettings.bat"

echo "--- Processing proxy settings ---"
getProxySettings "${CLUSTER_DIR}/workspace/bin/proxySettings.bat"

echo "--- Fetching IDM Tomcat service memory configuration ---"
MEM=`awk 'match($0,/-Xmx([0-9]+)[mM]/,m) {print m[1]}' ${INSTANCE_SETENV}`
if test -n "$MEM" ; then
    echo "$(( $MEM * 1024 ))k " >"${CLUSTER_DIR}/conf/tomcat.mem"
fi

if test -e  /usr/local/horizon/conf/domain_krb.properties; then
    cp /usr/local/horizon/conf/domain_krb.properties "${CLUSTER_DIR}/conf"
fi

echo "--- Generating data bundle ---"
cd "${CLUSTER_DIR}"
zip -qr "${ZIP_FILE_NAME_WITH_PATH}" *
cd ${SCRIPT_DIR}

echo "--- Encrypting data bundle ---"
export CLUSTER_PASSWORD
${JAVA_HOME}/bin/java -cp ${BC_JAR}:${FILE_ENC_JAR} com.vmware.horizon.common.tools.FileEncrypter -e -i "${ZIP_FILE_NAME_WITH_PATH}" -o "${ENCRYPTED_ZIP_FILE}" -E "CLUSTER_PASSWORD"
if test $? -ne 0; then
  echo "Failed to encrypt cluster file."
  cleanup 2
fi

echo
echo "--- Cluster file generation complete ---"
echo     ${ENCRYPTED_ZIP_FILE}

cleanup 0

